#!/usr/bin/env python

#****************************************************************************
#** $Id: sqltable4.py,v 1.1 2002/06/22 00:23:21 frispete Exp $
#**
#** Copyright (C) 1992-1998 Troll Tech AS.  All rights reserved.
#**
#** This file is part of an example program for PyQt.  This example
#** program may be used, distributed and modified without limitation.
#**
#*****************************************************************************/

import sys
from qt import *
from qtsql import *

TRUE  = 1
FALSE = 0

#DB_QTDRIVER = "QPSQL7"
DB_QTDRIVER = "QMYSQL3"
DB_HOSTNAME = "server"
DB_DATABASE = "testdb"
DB_USERNAME = "xy"
DB_PASSWORD = "zzz"

class CustomTable(QDataTable):
    def __init__(self, cursor, autoPopulate = FALSE, parent = None, name = None):
        QDataTable.__init__(self, cursor, autoPopulate, parent, name)

    def paintField(self, p, field, cr, b):
        if not field:
            return
        if str(field.name()) in ("salary", "monsalary"):
            v = field.value().toDouble()
            if v < 0:
                p.setPen(QColor("red"))
            value = QString(u"%.2f \u20ac" % v)
            #print unicode(value).encode("iso-8859-15")
            p.drawText(2, 2, cr.width()-6, cr.height()-4,
                        Qt.AlignRight|Qt.AlignVCenter, value)
        elif str(field.name()) == "statusid":
            query = QSqlQuery("SELECT name FROM status WHERE id=%s" %
                              field.value().toString())
            value = ""
            if query.next():
                value = query.value(0).toString()
            p.drawText(2, 2, cr.width()-4, cr.height()-4,
                        self.fieldAlignment(field), value)
        else:
            QDataTable.paintField(self, p, field, cr, b)


class StatusPicker(QComboBox):
    def __init__(self, parent = None, name = None):
        QComboBox.__init__(self, parent, name)
        cur = QSqlCursor("status")
        cur.select(cur.index("id"))
        while cur.next():
            self.insertItem(cur.value("name").toString(), cur.value("id").toInt())

class CustomSqlEditorFactory(QSqlEditorFactory):
    def __init__(self):
        QSqlEditorFactory.__init__(self)

    def createEditor(self, parent, field):
        try:
            if str(field.name()) == "statusid":
                return StatusPicker(parent)
        except AttributeError:
            pass
        return QSqlEditorFactory.createEditor(self, parent, field)

class StaffCursor(QSqlCursor):
    def __init__(self):
        QSqlCursor.__init__(self, "staff")
        monSalary = QSqlFieldInfo("monsalary", QVariant.Double)
        self.append(monSalary)
        self.setCalculated(monSalary.name(), TRUE)

    def calculateField(self, name):
        if str(name) == "monsalary":
            print self.value("salary").toDouble()
            v = self.value("salary").toDouble() / 12
            print v, type(v)
            print QVariant(v)
            print QVariant(v).toDouble()    # shouldn't this be transparent? => v
            return QVariant(v)
        return QVariant(QString.null)

class Table(CustomTable):
    def __init__(self):
        #self.staffCursor = QSqlCursor("staff")
        self.staffCursor = StaffCursor()
        QDataTable.__init__(self, self.staffCursor)
        self.propMap = QSqlPropertyMap()
        self.editorFactory = CustomSqlEditorFactory()
        self.propMap.insert("StatusPicker", "statusid")
        self.installPropertyMap(self.propMap)
        self.installEditorFactory(self.editorFactory)

        self.addColumn("forename", "Forename")
        self.addColumn("surname",  "Surname" )
        self.addColumn("salary",   "Annual Salary")
        self.addColumn("monsalary","Monthly Salary")
        self.addColumn("statusid", "Status")
        self.order = QStringList("surname")
        self.order.append("forename")
        #self.order = QStringList("id")
        self.setSort(self.order)
        self.refresh()


def createConnection():
    db = QSqlDatabase.addDatabase(DB_QTDRIVER)
    if db:
        db.setHostName(DB_HOSTNAME)
        db.setDatabaseName(DB_DATABASE)
        db.setUserName(DB_USERNAME)
        db.setPassword(DB_PASSWORD)
        if db.open():
            return TRUE
        else:
            print "Could not open testdb database"
            print db.lastError().driverText()
            print db.lastError().databaseText()
    else:
        print "Could not activate database driver"
    return FALSE

if __name__=='__main__':
    app = QApplication( sys.argv )

    if createConnection():
        t = Table()
        app.setMainWidget(t)
        t.resize(600, 250)
        t.show()
        app.exec_loop()


